@@ -5,7 +5,7 @@ use warnings;
 package MooX::Press;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.050';
+our $VERSION   = '0.051';
 
 use Types::Standard 1.008003 -is, -types;
 use Types::TypeTiny qw(ArrayLike HashLike);
@@ -164,13 +164,16 @@ sub import {
 		);
 	}
 	
-	{
+	if (defined $opts{'factory_package'}) {
+		no strict 'refs';
+		
 		my %methods;
 		my $method_installer = $opts{toolkit_install_methods} || ("install_methods");
 		
 		%methods = delete($opts{factory_package_can})->$_handle_list_add_nulls;
-		$methods{qualify} ||= sub { $builder->qualify($_[1], $opts{'prefix'}) };
-		$builder->$method_installer($opts{'factory_package'}||$opts{'caller'}, \%methods) if keys %methods;
+		$methods{qualify} ||= sub { $builder->qualify($_[1], $opts{'prefix'}) }
+			unless exists &{$opts{'factory_package'}.'::qualify'};
+		$builder->$method_installer($opts{'factory_package'}, \%methods) if keys %methods;
 		
 		%methods = delete($opts{type_library_can})->$_handle_list_add_nulls;
 		$builder->$method_installer($opts{type_library}, \%methods) if keys %methods;
@@ -277,8 +280,11 @@ sub croak {
 
 my $none;
 sub prepare_type_library {
+	no strict 'refs';
+	no warnings 'once';
 	my $builder = shift;
 	my ($lib, %opts) = @_;
+	return if exists &{"$lib\::_mooxpress_add_type"};
 	my ($version, $authority) = ($opts{version}, $opts{authority});
 	my %types_hash;
 	require Type::Tiny::Role;
@@ -330,12 +336,10 @@ sub prepare_type_library {
 				sub get_type_for_package { shift->type_library->get_type_for_package(@_) };
 				1;
 			',
-			$opts{'factory_package'}||$opts{'caller'},
+			$opts{'factory_package'},
 			B::perlstring($lib),
 		) or $builder->croak("Could not install type library methods into factory package: $@");
 	}
-	no strict 'refs';
-	no warnings 'once';
 	*{"$lib\::_mooxpress_add_type"} = $adder;
 	*{"$lib\::get_type_for_package"} = $getter;
 	${"$lib\::VERSION"} = $version if defined $version;
@@ -551,8 +555,6 @@ sub _make_package {
 	my $qname = $builder->qualify_name($name, $opts{prefix}, @isa);
 	my $tn = $builder->type_name($qname, $opts{prefix});
 	
-	print "$name -> $qname\n";
-	
 	if (!exists $opts{factory}) {
 		$opts{factory} = $opts{abstract} ? undef : sprintf('new_%s', lc $tn);
 	}
@@ -630,10 +632,14 @@ sub _make_package {
 		my %methods;
 		%methods = $opts{can}->$_handle_list_add_nulls;
 		$builder->$method_installer($qname, \%methods) if keys %methods;
-		%methods = $opts{factory_package_can}->$_handle_list_add_nulls;
-		$builder->$method_installer($opts{'factory_package'}||$opts{'caller'}, \%methods) if keys %methods;
-		%methods = $opts{type_library_can}->$_handle_list_add_nulls;
-		$builder->$method_installer($opts{type_library}, \%methods) if keys %methods;
+		if (defined $opts{factory_package}) {
+			%methods = $opts{factory_package_can}->$_handle_list_add_nulls;
+			$builder->$method_installer($opts{factory_package}, \%methods) if keys %methods;
+		}
+		if (defined $opts{type_library}) {
+			%methods = $opts{type_library_can}->$_handle_list_add_nulls;
+			$builder->$method_installer($opts{type_library}, \%methods) if keys %methods;
+		}
 	}
 	
 	{
@@ -868,8 +874,8 @@ sub _make_package {
 			});
 		}
 		
-		if (defined $opts{'factory_package'} or not exists $opts{'factory_package'}) {
-			my $fpackage = $opts{'factory_package'} || $opts{'caller'};
+		if (defined $opts{'factory_package'}) {
+			my $fpackage = $opts{'factory_package'};
 			if ($opts{'factory'}) {
 				my @methods = $opts{'factory'}->$_handle_list;
 				if ($opts{abstract} and @methods) {
